package ColonSeparated;

use strict;
use ConfigPatch;
use SpaceSeparated;
use Carp;
use StackTrace;

use vars qw/@ISA/;

@ISA=qw(SpaceSeparated);

sub new {
  my ($type, @rest) = @_;
  my($class) = ref($type) || $type || "ColonSeparated";

  my $this = $class->SUPER::new();
  bless $this, $class;

  if(@_ >= 2) {
    $this->readSubst($_[1]);
  }

  return $this;
}

##
# Determines whether this line is a setting
#
# Parameters
# $context  1 if parsing substitution pattern
#           0 if parsing config file to be modified
# $line     the input line
#
# Returns:
#  - an opaque key identifying the setting
#  - an opaque  descriptor, that will later we passed the same way to
#    subsSetting
#
# In this particular case, the key has the following format:
#  define.setting   = a define(`setting') line
#  feature.name     = a FEATURE(`name', `value') line
#  base.xyz         = a XYZ(`value') line
#
# The returned descriptor is as follows
#   [ isEnabled, value, trailer ]
sub isSetting {
  my ($this, $context, $line) = @_;

  if($line =~ /^(\#)?\s*([\/a-zA-Z0-9._]+)(:.*)/s) {
    return ($2, [ $1, $2, $3 ]);
  } else {
    return (undef, undef);
  }
}

1;
