package Controller;

use strict;
use English;
use ConfigSubs;

use MailMc;
use RcConfig;
use NamedConf;
use XinetdConf;
use Samba;
use SpaceSeparated;
use ColonSeparated;
use ItemList;
use Create;
use FileHandle;
use ModulesConf;
use SquidConf;
use Dovecot;

sub doFile {
  my ($base, $target, $file) = @_;

  open(IN, "<$base/$file") ||
    die "Could not read $base/$file ($ERRNO)\n";

  my $line = <IN>;
  close(IN);
  if($line !~ /^@@%(\S*)/) {
    die "First line of $base/$file must be @@%\n";
  }
  my $type = $1;
  my $subst;
  if($type eq "MailMc") {
    $subst = new MailMc("$base/$file");
  } elsif($type eq "RcConfig") {
    $subst = new RcConfig("$base/$file");
  } elsif($type eq "NamedConf") {
    $subst = new NamedConf("$base/$file");
  } elsif($type eq "XinetdConf") {
    $subst = new XinetdConf("$base/$file");
  } elsif($type eq "Samba") {
    $subst = new Samba("$base/$file");
  } elsif($type eq "SpaceSeparated") {
    $subst = new SpaceSeparated("$base/$file");
  } elsif($type eq "ColonSeparated") {
    $subst = new ColonSeparated("$base/$file");
  } elsif($type eq "ItemList") {
    $subst = new ItemList("$base/$file");
  } elsif($type eq "ModulesConf") {
    $subst = new ModulesConf("$base/$file");
  } elsif($type eq "SquidConf") {
    $subst = new SquidConf("$base/$file");
  } elsif($type eq "Dovecot") {
    $subst = new Dovecot("$base/$file");
  } elsif($type eq "Create") {
    $subst = new Create("$base/$file");
  } else {
    die "Unknown type $type\n";
  }
  $subst->doSubst("$target/$file");
}


sub doDir {
  my ($base, $target, $dirName, $mode) = @_;

  my $dir;

  $dirName =~ s/^\.\///;

  $dir = FileHandle->new;
  opendir($dir, "$base/$dirName") ||
    die "Could not open $dirName ($ERRNO)\n";
  while( defined (my $name = readdir($dir))) {
    doEntry($base, $target, $dirName, $mode, $name);
  }
}


sub doEntry {
    my ($base, $target, $dirName, $mode, $name) = @_;
    my $cName = "$dirName/$name";
    if ($name eq "." || $name eq "..") {
      # do nothing
      next;
    } elsif (-d "$base/$cName" && $cName ne "CVS") {
      mkdir "$target/$cName", 0755;
      eval {
	doDir($base, $target, $cName, $mode);
      };
      if($@) {
 	chomp($@);
	print STDERR "Error \"$@\" while doing directory \"$cName\" in \"$base\"\n";
      }
    } elsif (-f "$base/$cName") {
      eval {
	doFile($base, $target, $cName);
	};
      if($@) {
 	chomp($@);
	print STDERR "Error \"$@\" while doing file \"$cName\" line $main::lineNo in \"$base\"\n";
      }
    } else {
      die "$cName not found\n";
    }
}

1;
