package Dovecot;

use strict;
use ConfigPatch;
use Carp;
use StackTrace;

use vars qw/@ISA/;

@ISA=qw(RcConfig);

# To be verified... conservative choice
sub keyIsUnique {
    return 0;
}

sub new {
  my ($type, @rest) = @_;
  my($class) = ref($type) || $type || "Dovecot";

  my $this = $class->SUPER::new(@rest);
  bless $this, $class;

  return $this;
}


##
# Determines whether this is the start of a block
# Parameters
# $this
# $line       - the line to analyze
#
# Returns:
#  (undef,undef)	if not start of a block
#  (blockId,desc)	if start of a block. BlockId is a unique identifier
#			for the block (within its parent block), whereas
#			desc is additional, non-identifying data
sub isBlockStart {
  my ($this, $line) = @_;

  if($line =~ /^\s*([^ #]\S*)\s+(\S+)\s*\{\s*\#\s*ID\s*(\S+)/) {
    ("$1-$2-$3", $line);
  } elsif($line =~ /^\s*([^ #]\S*)\s+(\S+)\s*\{/) {
    ("$1-$2", undef);
  } elsif($line =~ /^\s*([^ #]\S*)\s*\{/) {
    ("$1", undef);
  } else {
    (undef, undef);
  }
}


##
# Determines whether this is the end of a block
# Parameters
# $this
# $line       - the line to analyze
# $blockId    - the opaque identifier of the block: allows to detect bad
#             nesting
#
# Returns:
#  key if end
#  undef otherwise
sub isBlockEnd {
  my ($this, $line, $blockId) = @_;
  if($line =~ /^\s*\}\s*/) {
    ("",$line);
  } else {
    undef;
  }
}


##
# Block auto end: if true, blocks automatically end at the start of the next
# block
sub blockAutoEnd {
  0;
}
