package ModulesConf;

use strict;
use ConfigPatch;
use Carp;
use StackTrace;

use vars qw/@ISA/;

@ISA=qw(ConfigPatch);

sub new {
  my ($type, @rest) = @_;
  my($class) = ref($type) || $type || "ModulesConf";

  my $this = $class->SUPER::new();
  bless $this, $class;

  if(@_ >= 2) {
    $this->readSubst($_[1]);
  }

  return $this;
}

##
# Answers whether this is a comment
# Parameters
# $line - the line which is a comment or not
# Returns:
# The input line if it is indeed a comment, else undef
sub isComment {
  my ($this, $line) = @_;
  if($line =~ /^(\s*\#)/) {
    return $line;
  } else {
    return undef;
  }
}

##
# Determines whether this line is a setting
#
# Parameters
# $context  1 if parsing substitution pattern
#           0 if parsing config file to be modified
# $line     the input line
#
# Returns:
#  - an opaque key identifying the setting
#  - an opaque  descriptor, that will later we passed the same way to
#    subsSetting
#
# In this particular case, the key has the following format:
#  define.setting   = a define(`setting') line
#  feature.name     = a FEATURE(`name', `value') line
#  base.xyz         = a XYZ(`value') line
#
# The returned descriptor is as follows
#   [ isEnabled, value, trailer ]
sub isSetting {
  my ($this, $context, $line) = @_;

  if($line =~ /^(\#\#)?\s*alias\s+([a-zA-Z0-9._]+)\s+(.*)/) {
    return ("alias.".$2, [ $1, $3, $4]);
  } elsif($line =~ /^(\#\#)?\s*options\s+([a-zA-Z0-9._]+)\s+(.*)/) {
    return ("options.".$2, [ $1, $3, $4]);
  } else {
    return (undef, undef);
  }
}

##
# Substitutes setting
#
# Parameters:
# $this     - this object
# $key      - opaque substitution key
# $orig     - original line (in file to be patched)
# $subst    - the opaque substitution descriptor, as returned by isSetting
# $comments - any comments preceding the substitution (added for new insertion
#             to be re-output for new settings
# Returns:
#  The newly substituted line
sub subsSetting {
  my ($this, $key, $orig, $subst, $accum) = @_;
  if(!defined $subst) {
    printStackTrace;
    croak("Subst not defined\n");
  }
#  print "Subst=".join(":", @$subst)."\n";
  my ($disabled, $value, $trailer) = (@$subst);
  my $middle;
  if($key =~ /alias\.(.*)/) {
    $middle="alias $1 ";
  } elsif($key =~ /options\.(.*)/) {
    $middle="options $1 ";
  } else {
    die "Bad key $key\n";
  }
  $disabled.$middle.$value."\n";
}

1;

